/*
    AllegroBobble - An implementation of the classic game using the Allegro
    library (version 4) and the Allegro Wrapper
    Copyright (C) 2013, Alessandro Francesco De Rosis

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

# include "Ball.h"
# include <sstream>
# include "PuzzleBubble.h"
# include <cmath>
# include <typeinfo>

Ball::Ball(PuzzleBubble& game_, const Point& center) :
       Sprite(imageName(game_.level), Point(center.getX() , center.getY())),
       game(game_), hit(false),boom("boom.wav",170)
{
   assert(getTipo()!=NULLO);
   t=getTipo();
}

Ball::Ball(bool hit_, PuzzleBubble& game_, const Point& center) :
       Sprite(imageName(game_.level), Point(center.getX() , center.getY())),
       game(game_), hit(hit_),boom("boom.wav",170)
{
       t=NULLO;
}


Ball::Ball(PuzzleBubble& _game, const Point& center, string imageName) :Sprite(imageName,Point(center.getX(),center.getY())),game(_game),hit(false), boom("boom.wav",170)
{
   assert(getTipo()!=NULLO);
   t=getTipo();
}


bool Ball:: alive() const
{    
     if(hit)
       return false;
     return true;
}

Tipo Ball:: getTipo() const
{
    if(getFileName()=="sfera1.bmp")
      return ROSSA;
    if(getFileName()=="sfera2.bmp")
      return BLU;
    if(getFileName()=="sfera3.bmp")
      return GIALLA;
    if(getFileName()=="sfera4.bmp")
      return VERDE;
    if(getFileName()=="sfera5.bmp")
      return VIOLA;
    return NULLO;
  
}
string Ball::imageName(int level) const
{
    assert(level>=1 && level<=5);
    int num_pallina=1;
    switch(level)
    {
    case 4:
        num_pallina=rand() % 4 +1;
        break;
    case 5:
        num_pallina=rand() % 5 +1;
        break;
    default: 
        num_pallina=rand() % 3 +1;
        break;
    }
    stringstream ss;
    ss << "sfera" << num_pallina << ".bmp";
    return ss.str();
}

void Ball::lancia()
{
     int a=game.freccia.get1X()-game.freccia.get2X();
     int b=game.freccia.get1Y()-game.freccia.get2Y();
    for(double t=0;!urto(game.daInserire.front());t+=0.06)
    {
       
      if(toccaPareteSinistra(game.daInserire.front()))
      {
         lancioDaPareteDiSinistra(a,b);
         break;
      }
      else if(toccaPareteDestra(game.daInserire.front()))
      {
         lancioDaPareteDiDestra(a,b);
         break;
      }
      else if(!toccaPareteSinistra(game.daInserire.front()) && !toccaPareteDestra(game.daInserire.front()))
         game.daInserire.front()->setLeftTopCorner(Point(game.freccia.get2X()-a*t-30,game.freccia.get2Y()-b*t-30));
       game.draw();
     }
      game.inserite.push_front(game.daInserire.front());
      game.daInserire.pop();
      scoppio();
      if(game.daInserire.size()==1)
         game.aggiungiADaInserire();
      
      game.middleLayer.add(DrawablePointer(*(game.daInserire.front())));

      
}

void Ball::lancioDaPareteDiSinistra(int old_a,int old_b)
{
   int a=-old_a;
   int b=old_b;
   game.daInserire.front()->setLeftTopCorner(Point(getLeftTopCorner().getX()+dammiRestoDallePareti(game.daInserire.front()),getLeftTopCorner().getY())); 
   int primoPuntoX=game.daInserire.front()->getLeftTopCorner().getX();
   int primoPuntoY=game.daInserire.front()->getLeftTopCorner().getY();
   
   for(double t=0;!urto(game.daInserire.front());t+=0.06)
   {
      game.daInserire.front()->setLeftTopCorner(Point(primoPuntoX+b*t,primoPuntoY+a*t));
      if(toccaPareteSinistra(game.daInserire.front()))
          lancioDaPareteDiSinistra(a,b);
      if(toccaPareteDestra(game.daInserire.front()))
          lancioDaPareteDiDestra(a,b);
      game.draw();
   }
}

void Ball::lancioDaPareteDiDestra(int old_a,int old_b)
{
   int a=old_a;
   int b=-old_b;
   game.daInserire.front()->setLeftTopCorner(Point(getLeftTopCorner().getX()+dammiRestoDallePareti(game.daInserire.front()),getLeftTopCorner().getY())); 
   int primoPuntoX=game.daInserire.front()->getLeftTopCorner().getX();
   int primoPuntoY=game.daInserire.front()->getLeftTopCorner().getY();
   
   for(double t=0;!urto(game.daInserire.front());t+=0.06)
   {
      game.daInserire.front()->setLeftTopCorner(Point(primoPuntoX+b*t,primoPuntoY+a*t));
      if(toccaPareteSinistra(game.daInserire.front()))
          lancioDaPareteDiSinistra(a,b);
      if(toccaPareteDestra(game.daInserire.front()))
          lancioDaPareteDiDestra(a,b);
      game.draw();
   }
}

bool Ball::toccaPareteSinistra(Ball* c) const 
{
   if(c->getLeftTopCorner().getX()<=30)
     return true;
   return false;
}

bool Ball::toccaPareteDestra(Ball* c) const 
{
    if(c->getLeftTopCorner().getX()>=390)
       return true;
    return false;
}

bool Ball::toccaPareteInAlto(Ball* c) const 
{
   if(c->getLeftTopCorner().getY()<=game.y_muro)
     return true;
   return false;
}

int Ball::dammiRestoDallePareti(Ball* c) const 
{
  if(toccaPareteSinistra(c))
    return 31%getLeftTopCorner().getX();
  if(toccaPareteDestra(c))
    return -getLeftTopCorner().getX()%389;
  return 0;
}
 
void Ball::impostaPallinaDopoUrto( Ball* da_set,const Ball* urtata,bool &gia_impostato)
{
    
    gia_impostato=true;
    enum allineamento{SINISTRA_Y=0,DESTRA_Y,DESTRA_X,SINISTRA_X};
    allineamento a;
    if(urtata->getLeftTopCorner().getX()+60- da_set->getLeftTopCorner().getX()<=50)
      a=DESTRA_Y;
    if(urtata->getLeftTopCorner().getX()+60- da_set->getLeftTopCorner().getX()>50)
      a=SINISTRA_Y;
    if(da_set->getLeftTopCorner().getY()- urtata->getLeftTopCorner().getY()<=18 && urtata->getLeftTopCorner().getX()+60- da_set->getLeftTopCorner().getX()<=50)
    {
      a=DESTRA_X;
    }
    if(da_set->getLeftTopCorner().getY()- urtata->getLeftTopCorner().getY()<=18 && urtata->getLeftTopCorner().getX()+60- da_set->getLeftTopCorner().getX()>50)
    {
      a=SINISTRA_X;
    }
    
    Ball b(game,Point(urtata->getLeftTopCorner().getX()-59, urtata->getLeftTopCorner().getY()));
    if(a==SINISTRA_X)
    {
       if(!urto(&b)  && !toccaPareteSinistra(&b))
         da_set->setLeftTopCorner(Point(urtata->getLeftTopCorner().getX()-60, urtata->getLeftTopCorner().getY()));
       else if(a==SINISTRA_X && (toccaPareteSinistra(&b) || urto(&b)))   
         a=SINISTRA_X;
    }
    if(a==DESTRA_X)
    {
       b.setLeftTopCorner(Point(urtata->getLeftTopCorner().getX()+59, urtata->getLeftTopCorner().getY()));
       if(!urto(&b)  && !toccaPareteDestra(&b))
         da_set->setLeftTopCorner(Point(urtata->getLeftTopCorner().getX()+60, urtata->getLeftTopCorner().getY()));
       else if(a==DESTRA_X && (toccaPareteDestra(&b) || urto(&b)))   
         a=DESTRA_Y;
    }
    if(a==SINISTRA_Y)
    {
         b.setLeftTopCorner(Point(urtata->getLeftTopCorner().getX()-29, urtata->getLeftTopCorner().getY()+50));
         if(!urto(&b)  && !toccaPareteSinistra(&b))
         {
           da_set->setLeftTopCorner(Point(urtata->getLeftTopCorner().getX()-30, urtata->getLeftTopCorner().getY()+50));
         }
         else if(a==SINISTRA_Y && (toccaPareteSinistra(&b) || urto(&b)))
         {
           da_set->setLeftTopCorner(Point(urtata->getLeftTopCorner().getX()+30, urtata->getLeftTopCorner().getY()+50));
         }
    }
    if (a==DESTRA_Y)
    { 
        b.setLeftTopCorner(Point(urtata->getLeftTopCorner().getX()+29, urtata->getLeftTopCorner().getY()+50));
        if(!toccaPareteDestra(&b) && !urto(&b))
        {
           da_set->setLeftTopCorner(Point(urtata->getLeftTopCorner().getX()+30, urtata->getLeftTopCorner().getY()+50));
        }
        else if(urto(&b) || toccaPareteDestra(&b))
        {
           da_set->setLeftTopCorner(Point(urtata->getLeftTopCorner().getX()-30, urtata->getLeftTopCorner().getY()+50));
        }
           
    }
    gia_impostato=false;
}

bool Ball::urto(Ball* c) 
{
    for(list<Ball*> ::iterator i=game.inserite.begin();i!=game.inserite.end();i++)
    {
       int dist=distanza(c,(*i));
       if(dist<=40)
       {
           static bool gia_impostato=false;
           if(!gia_impostato)
             impostaPallinaDopoUrto(c,(*i),gia_impostato);
          return true;
       }
       
    }
    return false;
}

queue<Ball*>  Ball::collegamenti_generici(Ball* c) const
{
    queue<Ball*> daVisitare;
    queue<Ball*> visitati;
    daVisitare.push(c);
    while(!daVisitare.empty())
    {
        bool visitato=ricerca(visitati, daVisitare.front());
        if(!visitato)
          visitati.push(daVisitare.front());
        daVisitare.pop();
        if(!visitato)
        {
          for(list<Ball*>:: iterator i=game.inserite.begin();i!=game.inserite.end();i++)
          {
              if(distanza(visitati.back(),(*i))<=60)
                 if(!ricerca(visitati,(*i)))
                    daVisitare.push(*i);
          }
        }
    }
    return visitati;
}
bool Ball::check_collegamento(Ball* c) const
{
    queue<Ball*> visitate=collegamenti_generici(c);
    while(!visitate.empty())
    {
          if((visitate.front())->getLeftTopCorner().getY()<c->getLeftTopCorner().getY())
              return true;
          visitate.pop();
    }
    return false;
}

int Ball:: distanza(Ball* b, Ball* c) const
{
     int distanza=(((c->getLeftTopCorner().getY()- (b)->getLeftTopCorner().getY())*(c->getLeftTopCorner().getY() - (b)->getLeftTopCorner().getY()))+((c->getLeftTopCorner().getX()-(b)->getLeftTopCorner().getX())*(c->getLeftTopCorner().getX()-(b)->getLeftTopCorner().getX())));
     return sqrt(distanza);
}


bool Ball::ricerca(queue<Ball*> visitati, Ball* b) const
{
     while(!visitati.empty())
     {
         if(*b==*visitati.front())
            return true;
       visitati.pop();
     }
     return false;
}
void Ball::scoppio()
{
    queue<Ball*> daVisitare;
    queue<Ball*> visitati;
    daVisitare.push(game.inserite.front());
    int punti=0;
    while(!daVisitare.empty())
    {
        bool visitato=ricerca(visitati, daVisitare.front());
        if(!visitato)
          visitati.push(daVisitare.front());
        daVisitare.pop();
        if(!visitato)
        {
           list<Ball*>:: iterator fine=game.inserite.begin();
          for(unsigned conta=0;conta<game.inserite.size()-6;conta++)
              fine++;
          for(list<Ball*>:: iterator i=game.inserite.begin();i!=fine;i++)
          {
              if(distanza(visitati.back(),(*i))<=60 && visitati.back()->getTipo()== (*i)->getTipo())
                 if(!ricerca(visitati,(*i)))
                    daVisitare.push(*i);
          }
        }
    }
    if(visitati.size()>=3)
    {
      list<Ball*> da_cancellare;  
      while(!visitati.empty())
      {
          da_cancellare.push_back(visitati.front());
          visitati.pop();
      }
     boom.play();
     for(list<Ball*>:: iterator c=da_cancellare.begin();c!=da_cancellare.end();c++)
     {
        for(list<Ball*>::iterator i=game.inserite.begin();i!=game.inserite.end();i++)
            if(*i==*c)
            {
              punti+=10;
              (*i)->hit=true;
              i=game.inserite.erase(i);
              i--; 
            }
     }
     if(game.inserite.size()>6)
     {
        bool trovato=true;
        while(trovato)
        {
           trovato=false;
           list<Ball*>:: iterator fine=game.inserite.begin();
           for(unsigned conta=0;conta<game.inserite.size()-6;conta++)
                 fine++;
           for(list<Ball*>::iterator i=game.inserite.begin();i!=fine;i++)
           if(!check_collegamento(*i))
           {

               trovato=true;
               (*i)->hit=true;
               game.inserite.erase(i);
               i=game.inserite.begin();
               fine=game.inserite.begin();
               punti+=10;
               if(game.inserite.size()<=6)
                  break;
               for(unsigned conta=0;conta<game.inserite.size()-6;conta++)
                  fine++;
           }
        }
     }
    }
         
    for(int i=0;i<=punti;i++)
    {
      stringstream ss;
      game.point+=i;
      ss << "Point: " << game.point;
      game.pointText.setText(ss.str());
    }
  
} 
