/*
    AllegroBobble - An implementation of the classic game using the Allegro
    library (version 4) and the Allegro Wrapper
    Copyright (C) 2013, Alessandro Francesco De Rosis

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef COMPOSITE_SPRITE_H
#define COMPOSITE_SPRITE_H

#include "CompositeBitmap.h"
#include "Sprite.h"

class CompositeSprite : public TSprite<CompositeBitmap> {
public:
    CompositeSprite(const string& filename, const Point& leftTopCorner = Point());
    CompositeSprite(unsigned width, unsigned height, const Point& leftTopCorner = Point());
    CompositeSprite(unsigned width, unsigned height, const Color& background, const Point& leftTopCorner = Point());
    CompositeSprite(const CompositeSprite& init);

    CompositeSprite& setNotification(Notification type);
    CompositeSprite& unsetNotification(Notification type);
    bool hasNotification(Notification type) const;
    
    virtual void onKeyPressed(const Keyboard& keyboard, int keyCode);
    virtual void onKeyReleased(const Keyboard& keyboard, int keyCode);
    virtual void onMouseButtonPressed(const Mouse& mouse, Mouse::Button button);
    virtual void onMouseButtonReleased(const Mouse& mouse, Mouse::Button button);
    virtual void onMouseMove(const Mouse& mouse);
    virtual void onTick();
    
    virtual void draw(BITMAP* bitmap);
    
private:
    int notifications;
    
    void notifyKeyPressed(const Keyboard& keyboard, int keyCode);
    void notifyKeyReleased(const Keyboard& keyboard, int keyCode);
    void notifyMouseButtonPressed(const Mouse& mouse, Mouse::Button button);
    void notifyMouseButtonReleased(const Mouse& mouse, Mouse::Button button);
    void notifyMouseMove(const Mouse& mouse);
    void notifyTick();
};

#endif

